searchNodes=[{"ref":"Oban.html","title":"Oban","type":"module","doc":"Oban isn&#39;t an application and won&#39;t be started automatically. It is started by a supervisor that must be included in your application&#39;s supervision tree. All of your configuration is passed into the Oban supervisor, allowing you to configure Oban like the rest of your application. # confg/config.exs config :my_app, Oban, repo: MyApp.Repo, queues: [default: 10, events: 50, media: 20] # lib/my_app/application.ex defmodule MyApp.Application do @moduledoc false use Application alias MyApp.{Endpoint, Repo} def start(_type, _args) do children = [ Repo, Endpoint, {Oban, Application.get_env(:my_app, Oban)} ] Supervisor.start_link(children, strategy: :one_for_one, name: MyApp.Supervisor) end end Configuring Queues Queues are specified as a keyword list where the key is the name of the queue and the value is the maximum number of concurrent jobs. The following configuration would start four queues with concurrency ranging from 5 to 50: queues: [default: 10, mailers: 20, events: 50, media: 5] There isn&#39;t a limit to the number of queues or how many jobs may execute concurrently. Here are a few caveats and guidelines: Each queue will run as many jobs as possible concurrently, up to the configured limit. Make sure your system has enough resources (i.e. database connections) to handle the concurrent load. Only jobs in the configured queues will execute. Jobs in any other queue will stay in the database untouched. Be careful how many concurrent jobs make expensive system calls (i.e. FFMpeg, ImageMagick). The BEAM ensures that the system stays responsive under load, but those guarantees don&#39;t apply when using ports or shelling out commands. Creating Workers Worker modules do the work of processing a job. At a minimum they must define a perform/1 function, which is called with an args map. Define a worker to process jobs in the events queue: defmodule MyApp.Workers.Business do use Oban.Worker, queue: &quot;events&quot;, max_attempts: 10 @impl Oban.Worker def perform(%{&quot;id&quot; =&gt; id}) do model = MyApp.Repo.get(MyApp.Business.Man, id) IO.inspect(model) end end The return value of perform/1 doesn&#39;t matter and is entirely ignored. If the job raises an exception or throws an exit then the error will be reported and the job will be retried (provided there are attempts remaining). See Oban.Worker for more details. Enqueueing Jobs Jobs are simply Ecto strucs and are enqueued by inserting them into the database. Here we insert a job into the default queue and specify the worker by module name: %{id: 1, user_id: 2} |&gt; Oban.Job.new(queue: :default, worker: MyApp.Worker) |&gt; MyApp.Repo.insert() For convenience and consistency all workers implement a new/2 function that converts an args map into a job changeset suitable for inserting into the database: %{in_the: &quot;business&quot;, of_doing: &quot;business&quot;} |&gt; MyApp.Workers.Business.new() |&gt; MyApp.Repo.insert() The worker&#39;s defaults may be overridden by passing options: %{vote_for: &quot;none of the above&quot;} |&gt; MyApp.Workers.Business.new(queue: &quot;special&quot;, max_attempts: 5) |&gt; MyApp.Repo.insert() Jobs may be scheduled down to the second any time in the future: %{id: 1} |&gt; MyApp.Workers.Business.new(schedule_in: 5) |&gt; MyApp.Repo.insert() See Oban.Job.new/2 for a full list of job options. Testing Oban doesn&#39;t provide any special mechanisms for testing. However, here are a few recommendations for running tests in isolation. Set a high poll_interval in your test configuration. This effectively stops queues from polling and will prevent inserted jobs from executing. config :my_app, Oban, poll_interval: :timer.minutes(30) Be sure to use the Ecto Sandbox for testing. Oban makes use of database pubsub events to dispatch jobs, but pubsub events never fire within a transaction. Since sandbox tests run within a transaction no events will fire and jobs won&#39;t be dispatched. config :my_app, MyApp.Repo, pool: Ecto.Adapters.SQL.Sandbox Error Handling When a job raises an error or exits during execution the details are recorded within the errors array on the job. Provided the number of execution attempts is below the configured max_attempts, the job will automatically be retried in the future. The retry delay has a quadratic backoff, meaning the job&#39;s second attempt will be after 16s, third after 31s, fourth after 1m 36s, etc. Error Details Execution errors are stored as a formatted exception along with metadata about when the failure ocurred and which attempt caused it. Each error is stored with the following keys: at The utc timestamp when the error occurred at attempt The attempt number when the error ocurred error A formatted error message and stacktrace See the Instrumentation docs below for an example of integrating with external error reporting systems. Limiting Retries By default jobs will be retried up to 20 times. The number of retries is controlled by the max_attempts value, which can be set at the Worker or Job level. For example, to instruct a worker to discard jobs after three failures: use Oban.Worker, queue: &quot;limited&quot;, max_attempts: 3 Pruning Historic Jobs Job stats and queue introspection is built on keeping job rows in the database after they have completed. This allows administrators to review completed jobs and build informative aggregates, but at the expense of storage and an unbounded table size. To prevent the oban_jobs table from growing indefinitely, Oban provides active pruning of completed jobs. By default, pruning is disabled. To enable pruning we configure a supervision tree with the :prune option. There are three distinct modes of pruning: :disabled - This is the default, where no pruning happens at all {:maxlen, count} - Pruning is based on the number of rows in the table, any rows beyond the configured count will be deleted {:maxage, seconds} - Pruning is based on a row&#39;s age, any rows older than the configured number of seconds will be deleted. The age unit is always specified in seconds, but values on the scale of days, weeks or months are perfectly acceptable. Pruning is best-effort and performed out-of-band. This means that all limits are soft; jobs beyond a specified length or age may not be pruned immediately after jobs complete. Prune timing is based on the configured poll_interval, where pruning occurs once for every 60 queue polls. With the default poll_interval of 1 second that means pruning occurs at system startup and then once every minute afterwards. Note, only jobs in a completed or discarded state will be deleted. Currently executing jobs and older jobs that are still in the available state will be retained. Instrumentation Oban provides integration with Telemetry, a dispatching library for metrics. It is easy to report Oban metrics to any backend by attaching to :oban events. For exmaple, to log out the timing for all executed jobs: defmodule ObanLogger do require Logger def handle_event([:oban, :job, :executed], %{timing: timing}, meta, _config) do Logger.info(&quot;[meta.queue] meta.worker meta.event in timing&quot;) end end :telemetry.attach(&quot;oban-logger&quot;, [:oban, :job, :executed], &amp;ObanLogger.handle_event/4, nil) Another great use of execution data is error reporting. Here is an example of integrating with Honeybadger: defmodule ErrorReporter do def handle_event([:oban, :job, :failure], _timing, %{event: :failure} = meta, _config) do context = Map.take(meta, [:id, :args, :queue, :worker]) Honeybadger.notify(meta.error, context, meta.stack) end end :telemetry.attach(&quot;oban-errors&quot;, [:oban, :job, :executed], &amp;ErrorReporter.handle_event/4, nil) Here is a reference for metric event metadata: eventmetadata :success:id, :args, :queue, :worker :failure:id, :args, :queue, :worker, :kind, :error, :stack"},{"ref":"Oban.html#child_spec/1","title":"Oban.child_spec/1","type":"function","doc":"Returns a specification to start this module under a supervisor. See Supervisor."},{"ref":"Oban.html#config/0","title":"Oban.config/0","type":"function","doc":"Retreive the current config struct."},{"ref":"Oban.html#kill_job/1","title":"Oban.kill_job/1","type":"function","doc":"Kill an actively executing job and mark it as discarded, ensuring that it won&#39;t be retried. If the job happens to fail before it can be killed the state is set to discarded. However, if it manages to complete successfully then the state will still be completed. Example Kill a long running job with an id of 1: Oban.kill_job(1) :ok"},{"ref":"Oban.html#pause_queue/1","title":"Oban.pause_queue/1","type":"function","doc":"Pause a running queue, preventing it from executing any new jobs. All running jobs will remain running until they are finished. When shutdown begins all queues are paused. Example Pause the default queue: Oban.pause_queue(:default) :ok"},{"ref":"Oban.html#resume_queue/1","title":"Oban.resume_queue/1","type":"function","doc":"Resume executing jobs in a paused queue. Example Resume a paused default queue: Oban.resume_queue(:default) :ok"},{"ref":"Oban.html#scale_queue/2","title":"Oban.scale_queue/2","type":"function","doc":"Scale the concurrency for a queue. Example Scale a queue up, triggering immediate execution of queued jobs: Oban.scale_queue(:default, 50) :ok Scale the queue back down, allowing executing jobs to finish: Oban.scale_queue(:default, 5) :ok"},{"ref":"Oban.html#start_link/1","title":"Oban.start_link/1","type":"function","doc":"Starts an Oban supervision tree linked to the current process. Options :name — used for name supervisor registration :node — used to identify the node that the supervision tree is running in. If no value is provided it will use the node name in a distributed system, the hostname in an isolated node. See &quot;Node Name&quot; below. :repo — specifies the Ecto repo used to insert and retreive jobs. :queues — a keyword list where the keys are queue names and the values are the concurrency setting. For example, setting queues to [default: 10, exports: 5] would start the queues default and exports with a combined concurrency level of 20. The concurrency setting specifies how many jobs each queue will run concurrently. :poll_interval - the amount of time between a queue pulling new jobs, specified in milliseconds. This is directly tied to the resolution of scheduled jobs. For example, with a poll_interval of 5_000ms, scheduled jobs are checked every 5 seconds. The default is 1_000ms. :prune - configures job pruning behavior, see &quot;Pruning Historic Jobs&quot; for more information :shutdown_grace_period - the amount of time a queue will wait for executing jobs to complete before hard shutdown, specified in milliseconds. The default is 15_000, or 15 seconds. Note that any options passed to start_link will override options set through the using macro. Examples To start an Oban supervisor within an application&#39;s supervision tree: def start(_type, _args) do children = [MyApp.Repo, {Oban, queues: [default: 50]}] Supervisor.start_link(children, strategy: :one_for_one, name: MyApp.Supervisor) end Node Name When the node value hasn&#39;t been configured it will be generated based on the environment: In a distributed system the node name is used In a Heroku environment the system environment&#39;s DYNO value is used Otherwise, the system hostname is used"},{"ref":"Oban.html#t:option/0","title":"Oban.option/0","type":"type","doc":""},{"ref":"Oban.Job.html","title":"Oban.Job","type":"module","doc":"A Job is an Ecto schema used for asynchronous execution. Job changesets are created by your application code and inserted into the database for asynchronous execution. Jobs can be inserted along with other application data as part of a transaction, which guarantees that jobs will only be triggered from a successful transaction."},{"ref":"Oban.Job.html#new/2","title":"Oban.Job.new/2","type":"function","doc":"Construct a new job changeset ready for insertion into the database. Options :max_attempts — the maximum number of times a job can be retried if there are errors during execution :queue — a named queue to push the job into. Jobs may be pushed into any queue, regardless of whether jobs are currently being processed for the queue. :scheduled_in - the number of seconds until the job should be executed :scheduled_at - a time in the future after which the job should be executed :worker — a module to execute the job in. The module must implement the Oban.Worker behaviour. Examples Insert a job with the :default queue: %{id: 1, user_id: 2} |&gt; Oban.Job.new(queue: :default, worker: MyApp.Worker) |&gt; MyApp.Repo.insert() Generate a pre-configured job for MyApp.Worker and push it: %{id: 1, user_id: 2} |&gt; MyApp.Worker.new() |&gt; MyApp.Repo.insert() Schedule a job to run in 5 seconds: %{id: 1} |&gt; MyApp.Worker.new(scheduled_in: 5) |&gt; MyApp.Repo.insert()"},{"ref":"Oban.Job.html#t:args/0","title":"Oban.Job.args/0","type":"type","doc":""},{"ref":"Oban.Job.html#t:errors/0","title":"Oban.Job.errors/0","type":"type","doc":""},{"ref":"Oban.Job.html#t:option/0","title":"Oban.Job.option/0","type":"type","doc":""},{"ref":"Oban.Job.html#t:t/0","title":"Oban.Job.t/0","type":"type","doc":""},{"ref":"Oban.Worker.html","title":"Oban.Worker","type":"behaviour","doc":"Defines a behavior and macro to guide the creation of worker modules. Worker modules do the work of processing a job. At a minimum they must define a perform/1 function, which will be called with an args map. Defining Workers Define a worker to process jobs in the events queue: defmodule MyApp.Workers.Business do use Oban.Worker, queue: &quot;events&quot;, max_attempts: 10 @impl Oban.Worker def perform(args) do IO.inspect(args) end end The perform/1 function will always receive the jobs args map. In this example the worker will simply inspect any arguments that are provided. Note that the return value isn&#39;t important. If perform/1 returns without raising an exception the job is considered complete. Enqueuing Jobs All workers implement a new/2 function that converts an args map into a job changeset suitable for inserting into the database for later execution: %{in_the: &quot;business&quot;, of_doing: &quot;business&quot;} |&gt; MyApp.Workers.Business.new() |&gt; MyApp.Repo.insert() The worker&#39;s defaults may be overridden by passing options: %{vote_for: &quot;none of the above&quot;} |&gt; MyApp.Workers.Business.new(queue: &quot;special&quot;, max_attempts: 5) |&gt; MyApp.Repo.insert() See Oban.Job for all available options."},{"ref":"Oban.Worker.html#c:new/2","title":"Oban.Worker.new/2","type":"callback","doc":"Build a job changeset for this worker with optional overrides. See Oban.Job.new/2 for the available options."},{"ref":"Oban.Worker.html#c:perform/1","title":"Oban.Worker.perform/1","type":"callback","doc":"The perform/1 function is called when the job is executed. The function is passed a job&#39;s args, which is always a map with string keys. The return value is not important. If the function executes without raising an exception it is considered a success. If the job raises an exception it is a failure and the job may be scheduled for a retry."},{"ref":"readme.html","title":"README","type":"extras","doc":"Robust job processing in Elixir, backed by modern PostgreSQL. Reliable, observable and loaded with enterprise grade features."},{"ref":"readme.html#table-of-contents","title":"README - Table of Contents","type":"extras","doc":"Features Installation Usage Configuring Queues Creating Workers Enqueuing Jobs Testing Contributing"},{"ref":"readme.html#features","title":"README - Features","type":"extras","doc":"Oban&#39;s primary goals are reliability, consistency and observability. It is fundamentally different from other background job processing tools because it retains job data for historic metrics and inspection. You can leave your application running indefinitely without worrying about jobs being lost or orphaned due to crashes. Advantages over in-memory, mnesia, Redis and RabbitMQ based tools: Fewer Dependencies — If you are running a web app there is a very good chance that you&#39;re running on top of a RDBMS. Running your job queue within PostgreSQL minimizes system dependencies and Transactional Control — Enqueue a job along with other database changes, ensuring that everything is committed or rolled back atomically. Database Backups — Jobs are stored inside of your primary database, which means they are backed up together with the data that they relate to. Advanced features and advantages over other RDBMS based tools: Isolated Queues — Jobs are stored in a single table but are executed in distinct queues. Each queue runs in isolation, ensuring that a jobs in a single slow queue can&#39;t back up other faster queues. Queue Control — Queues can be paused, resumed and scaled independently at runtime. Job Killing — Jobs can be killed in the middle of execution regardless of which node they are running on. This stops the job at once and flags it as discarded. Triggered execution — Database triggers ensure that jobs are dispatched as soon as they are inserted into the database. Scheduled Jobs — Jobs can be scheduled at any time in the future, down to the second. Job Safety — When a process crashes or the BEAM is terminated executing jobs aren&#39;t lost—they are quickly recovered by other running nodes or immediately when the node is restarted. Historic Metrics — After a job is processed the row is not deleted. Instead, the job is retained in the database to provide metrics. This allows users to inspect historic jobs and to see aggregate data at the job, queue or argument level. Node Metrics — Every queue broadcasts metrics during runtime. These are used to monitor queue health across nodes. Queue Draining — Queue shutdown is delayed so that slow jobs can finish executing before shutdown. Telemetry Integration — Job life-cycle events are emitted via Telemetry integration. This enables simple logging, error reporting and health checkups without plug-ins."},{"ref":"readme.html#installation","title":"README - Installation","type":"extras","doc":"Oban is published on Hex. Add it to your list of dependencies in mix.exs: def deps do [ {:oban, &quot;~&gt; 0.2&quot;} ] end Then run mix deps.get to install Oban and its dependencies, including Ecto, Jason and Postgrex. After the packages are installed you must create a database migration to add the oban_jobs table to your database: mix ecto.gen.migration add_oban_jobs_table Open the generated migration in your editor and delegate the up and down functions to Oban.Migrations: defmodule MyApp.Repo.Migrations.AddObanJobsTable do use Ecto.Migration defdelegate up, to: Oban.Migrations defdelegate down, to: Oban.Migrations end Finally, run the migration to create the table: mix ecto.migrate Next see Usage for how to integrate Oban into your application and start defining jobs!"},{"ref":"readme.html#usage","title":"README - Usage","type":"extras","doc":"Oban isn&#39;t an application and won&#39;t be started automatically. It is started by a supervisor that must be included in your application&#39;s supervision tree. All of your configuration is passed into the Oban supervisor, allowing you to configure Oban like the rest of your application. # confg/config.exs config :my_app, Oban, repo: MyApp.Repo, queues: [default: 10, events: 50, media: 20] # lib/my_app/application.ex defmodule MyApp.Application do @moduledoc false use Application alias MyApp.{Endpoint, Repo} def start(_type, _args) do children = [ Repo, Endpoint, {Oban, Application.get_env(:my_app, Oban)} ] Supervisor.start_link(children, strategy: :one_for_one, name: MyApp.Supervisor) end end Configuring Queues Queues are specified as a keyword list where the key is the name of the queue and the value is the maximum number of concurrent jobs. The following configuration would start four queues with concurrency ranging from 5 to 50: queues: [default: 10, mailers: 20, events: 50, media: 5] There isn&#39;t a limit to the number of queues or how many jobs may execute concurrently. Here are a few caveats and guidelines: Each queue will run as many jobs as possible concurrently, up to the configured limit. Make sure your system has enough resources (i.e. database connections) to handle the concurrent load. Only jobs in the configured queues will execute. Jobs in any other queue will stay in the database untouched. Be careful how many concurrent jobs make expensive system calls (i.e. FFMpeg, ImageMagick). The BEAM ensures that the system stays responsive under load, but those guarantees don&#39;t apply when using ports or shelling out commands. Creating Workers Worker modules do the work of processing a job. At a minimum they must define a perform/1 function, which is called with an args map. Define a worker to process jobs in the events queue: defmodule MyApp.Workers.Business do use Oban.Worker, queue: &quot;events&quot;, max_attempts: 10 @impl Oban.Worker def perform(%{&quot;id&quot; =&gt; id}) do model = MyApp.Repo.get(MyApp.Business.Man, id) IO.inspect(model) end end The return value of perform/1 doesn&#39;t matter and is entirely ignored. If the job raises an exception or throws an exit then the error will be reported and the job will be retried (provided there are attempts remaining). Enqueueing Jobs Jobs are simply Ecto strucs and are enqueued by inserting them into the database. Here we insert a job into the default queue and specify the worker by module name: %{id: 1, user_id: 2} |&gt; Oban.Job.new(queue: :default, worker: MyApp.Worker) |&gt; MyApp.Repo.insert() For convenience and consistency all workers implement a new/2 function that converts an args map into a job changeset suitable for inserting into the database: %{in_the: &quot;business&quot;, of_doing: &quot;business&quot;} |&gt; MyApp.Workers.Business.new() |&gt; MyApp.Repo.insert() The worker&#39;s defaults may be overridden by passing options: %{vote_for: &quot;none of the above&quot;} |&gt; MyApp.Workers.Business.new(queue: &quot;special&quot;, max_attempts: 5) |&gt; MyApp.Repo.insert() Jobs may be scheduled down to the second any time in the future: %{id: 1} |&gt; MyApp.Workers.Business.new(schedule_in: 5) |&gt; MyApp.Repo.insert()"},{"ref":"readme.html#testing","title":"README - Testing","type":"extras","doc":"Oban doesn&#39;t provide any special mechanisms for testing. However, here are a few recommendations for running tests in isolation. Set a high poll_interval in your test configuration. This effectively stops queues from polling and will prevent inserted jobs from executing. config :my_app, Oban, poll_interval: :timer.minutes(30) Be sure to use the Ecto Sandbox for testing. Oban makes use of database pubsub events to dispatch jobs, but pubsub events never fire within a transaction. Since sandbox tests run within a transaction no events will fire and jobs won&#39;t be dispatched. config :my_app, MyApp.Repo, pool: Ecto.Adapters.SQL.Sandbox"},{"ref":"readme.html#contributing","title":"README - Contributing","type":"extras","doc":"To run the Oban test suite you must have PostgreSQL 10+ running locally with a database named oban_test. Follow these steps to create the database, create the database and run all migrations: # Create the database MIX_ENV=test mix ecto.create -r Oban.Test.Repo # Run the base migration MIX_ENV=test mix ecto.migrate -r Oban.Test.Repo To ensure a commit passes CI you should run mix ci locally, which executes the following commands: Check formatting (mix format --check-formatted) Lint with Credo (mix credo --strict) Run all tests (mix test) Run Dialyzer (mix dialyzer --halt-exit-status)"},{"ref":"changelog.html","title":"Changelog","type":"extras","doc":"Changelog All notable changes to this project will be documented in this file. The format is based on Keep a Changelog and this project adheres to Semantic Versioning."},{"ref":"changelog.html#0-2-0-2019-05-15","title":"Changelog - 0.2.0 - 2019-05-15","type":"extras","doc":"Added [Oban] Add pause_queue/2, resume_queue/2 and scale_queue/3 for dynamically controlling queues. [Oban] Add kill_job/2 for terminating running jobs and marking them as discarded. [Oban] Add config/0 for retreiving the supervisors config. This is primarily useful for integrating oban into external applications. [Oban.Queue.Producer] Use database triggers to immediately dispatch when a job is inserted into the oban_jobs table. [Oban.Job] Execution errors are stored as a jsonb array for each job. Each error is stored, not just the most recent one. Error entries contains these keys: at The utc timestamp when the error occurred at attempt The attempt number when the error ocurred error A formatted error message and stacktrace, passed through Exception.blame/3 [Oban.Config] Validate all options based on type and allowed values. Any invalid option will raise, preventing supervisor boot. [Oban.Notifier] Broadcast runtime gossip through pubsub, allowing any external system to get stats at the node and queue level. Changed [Oban.Queue.Supervisor] Set the min_demand to 1 for all consumers. This ensures that each queue will run the configured number of concurrent jobs. By default the min_demand is half of max_demand, which means a few slow jobs can prevent the queue from running the expected number of concurrent jobs. [Oban.Job] Change psuedo-states based on job properties into fixed states, this applies to scheduled and retryable. [Oban.Job] The &quot;Elixir&quot; prefix is stripped from worker names before storing jobs in the database. Module lookup performs the same way, but it cleans up displaying the worker name as a string. [Oban.Job] Accept all job fields as changeset parameters. While not encouraged for regular use, this is essential for testing various states."},{"ref":"changelog.html#0-1-0-2019-03-10","title":"Changelog - 0.1.0 - 2019-03-10","type":"extras","doc":"[Oban] Initial release with base functionality."}]